from .auth import RegisterSerializer, ResetPasswordSerializer
from .core import DatasetSerializer, CategorySerializer, MetadataSerialiazer
from .raster_file import RasterFileLayerSerializer
from .vector_file import PgVectorTableSerializer



from django.urls import path
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _
from wagtail_modeladmin.helpers import AdminURLHelper
from wagtail_modeladmin.views import CreateView, EditView, IndexView

from geomanager.admin.base import ModelAdminCanHide, BaseModelAdmin, LayerIndexView, LayerFileDeleteView
from geomanager.models import Dataset, VectorFileLayer, PgVectorTable, Category
from geomanager.views import (
    upload_vector_file,
    publish_vector,
    delete_vector_upload,
    preview_vector_layers
)


class VectorFileLayerCreateView(CreateView):
    def get_form(self):
        form = super().get_form()
        form.fields["dataset"].queryset = Dataset.objects.filter(layer_type="vector_file")

        dataset_id = self.request.GET.get("dataset_id")
        if dataset_id:
            initial = {**form.initial}
            initial.update({"dataset": dataset_id})
            form.initial = initial
        return form

    def get_context_data(self, **kwargs):
        context_data = super(VectorFileLayerCreateView, self).get_context_data(**kwargs)

        category_admin_helper = AdminURLHelper(Category)
        category_index_url = category_admin_helper.get_action_url("index")

        datasets_admin_helper = AdminURLHelper(Dataset)
        datasets_index_url = datasets_admin_helper.get_action_url("index")

        navigation_items = [
            {"url": category_index_url, "label": Category._meta.verbose_name_plural},
            {"url": datasets_index_url, "label": Dataset._meta.verbose_name_plural},
            {"url": "#", "label": _("New") + f" {VectorFileLayer._meta.verbose_name}"},
        ]

        context_data.update({
            "navigation_items": navigation_items,
        })

        return context_data


class VectorFileLayerEditView(EditView):
    def get_form(self):
        form = super().get_form()
        form.fields["dataset"].queryset = Dataset.objects.filter(layer_type="vector_file")
        return form

    def get_context_data(self, **kwargs):
        context_data = super(VectorFileLayerEditView, self).get_context_data(**kwargs)

        category_admin_helper = AdminURLHelper(Category)
        category_index_url = category_admin_helper.get_action_url("index")

        datasets_admin_helper = AdminURLHelper(Dataset)
        datasets_index_url = datasets_admin_helper.get_action_url("index")

        layer_admin_helper = AdminURLHelper(VectorFileLayer)
        layer_index_url = layer_admin_helper.get_action_url("index")

        navigation_items = [
            {"url": category_index_url, "label": Category._meta.verbose_name_plural},
            {"url": datasets_index_url, "label": Dataset._meta.verbose_name_plural},
            {"url": layer_index_url, "label": VectorFileLayer._meta.verbose_name_plural},
            {"url": "#", "label": self.instance.title},
        ]

        context_data.update({
            "navigation_items": navigation_items,
        })

        return context_data


class VectorFileLayerModelAdmin(BaseModelAdmin, ModelAdminCanHide):
    model = VectorFileLayer
    hidden = True
    exclude_from_explorer = True
    menu_label = _("Vector Layers")
    index_view_class = LayerIndexView
    create_view_class = VectorFileLayerCreateView
    edit_view_class = VectorFileLayerEditView
    list_display = ("title",)
    list_filter = ("dataset",)
    index_template_name = "geomanager/modeladmin/index_without_custom_create.html"

    def __init__(self, parent=None):
        super().__init__(parent)
        self.list_display = (list(self.list_display) or []) + ['dataset_link', "uploaded_files", "upload_files",
                                                               'preview_layer', 'mapviewer_map_url']
        self.dataset_link.__func__.short_description = _('Dataset')
        self.uploaded_files.__func__.short_description = _("View Uploaded Files")
        self.upload_files.__func__.short_description = _('Upload Vector Files')
        self.preview_layer.__func__.short_description = _('Preview on Map')
        self.mapviewer_map_url.__func__.short_description = _("View on MapViewer")

    def mapviewer_map_url(self, obj):
        label = _("MapViewer")
        button_html = f"""
                        <a href="{obj.mapviewer_map_url}" target="_blank" rel="noopener noreferrer" class="button button-small button--icon button-secondary">
                            <span class="icon-wrapper">
                                <svg class="icon icon-map icon" aria-hidden="true">
                                    <use href="#icon-map"></use>
                                </svg>
                            </span>
                            {label}
                        </a>
                    """
        return mark_safe(button_html)

    def dataset_link(self, obj):
        button_html = f"""
            <a href="{obj.dataset.dataset_url()}">
            {obj.dataset.title}
            </a>
        """
        return mark_safe(button_html)

    def upload_files(self, obj):
        disabled = "" if not obj.has_data_table else "disabled"

        label = _("Upload Files")
        button_html = f"""
            <a href="{obj.upload_url}" class="button button-small bicolor button--icon" {disabled}>
                <span class="icon-wrapper">
                    <svg class="icon icon-plus icon" aria-hidden="true">
                        <use href="#icon-upload"></use>
                    </svg>
                </span>
                {label}
            </a>
        """
        return mark_safe(button_html)

    def preview_layer(self, obj):
        label = _("Preview Layer")
        button_html = f"""
            <a href="{obj.preview_url}" class="button button-small button--icon button-secondary">
                <span class="icon-wrapper">
                    <svg class="icon icon-plus icon" aria-hidden="true">
                        <use href="#icon-view"></use>
                    </svg>
                </span>
                {label}
            </a>
        """
        return mark_safe(button_html)

    def uploaded_files(self, obj):
        label = _("Uploaded Files")
        button_html = f"""
            <a href="{obj.get_uploads_list_url()}" class="button button-small button--icon bicolor button-secondary">
                <span class="icon-wrapper">
                    <svg class="icon icon-list-ol icon" aria-hidden="true">
                        <use href="#icon-list-ol"></use>
                    </svg>
                </span>
                {label}
            </a>
        """
        return mark_safe(button_html)


class VectorFileIndexView(IndexView):
    def get_context_data(self, **kwargs):
        context_data = super(VectorFileIndexView, self).get_context_data(**kwargs)

        model_verbose_name = self.model._meta.verbose_name_plural

        category_admin_helper = AdminURLHelper(Category)
        categories_url = category_admin_helper.get_action_url("index")

        dataset_admin_helper = AdminURLHelper(Dataset)
        datasets_url = dataset_admin_helper.get_action_url("index")

        layer_admin_helper = AdminURLHelper(VectorFileLayer)
        layers_url = layer_admin_helper.get_action_url("index")

        navigation_items = [
            {"url": categories_url, "label": Category._meta.verbose_name_plural},
            {"url": datasets_url, "label": Dataset._meta.verbose_name_plural},
            {"url": layers_url, "label": VectorFileLayer._meta.verbose_name_plural},
            {"url": "#", "label": model_verbose_name},
        ]

        context_data.update({
            "navigation_items": navigation_items,
        })

        return context_data


class VectorTableModelAdmin(BaseModelAdmin, ModelAdminCanHide):
    model = PgVectorTable
    index_view_class = VectorFileIndexView
    hidden = True
    list_display = ("__str__", "table_name",)
    list_filter = ("layer",)
    index_template_name = "geomanager/modeladmin/index_without_custom_create.html"
    inspect_view_enabled = True
    delete_view_class = LayerFileDeleteView


urls = [
    path('upload-vector/', upload_vector_file, name='geomanager_upload_vector'),
    path('upload-vector/<uuid:dataset_id>/', upload_vector_file, name='geomanager_dataset_upload_vector'),
    path('upload-vector/<uuid:dataset_id>/<uuid:layer_id>/', upload_vector_file,
         name='geomanager_dataset_layer_upload_vector'),
    path('publish-vector/<int:upload_id>/', publish_vector, name='geomanager_publish_vector'),
    path('delete-vector-upload/<int:upload_id>/', delete_vector_upload, name='geomanager_delete_vector_upload'),
    path('preview-vector-layers/<uuid:dataset_id>/', preview_vector_layers, name='geomanager_preview_vector_dataset'),
    path('preview-vector-layers/<uuid:dataset_id>/<uuid:layer_id>/', preview_vector_layers,
         name='geomanager_preview_vector_layer'),
]



# Generated by Django 4.1.10 on 2023-09-20 08:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('geomanager', '0018_wmslayer_custom_get_capabilities_url'),
    ]

    operations = [
        migrations.AddField(
            model_name='wmslayer',
            name='get_capabilities_layer_name',
            field=models.CharField(blank=True, help_text='Alternative layer name for the GetCapabilities request', max_length=255, null=True, verbose_name='Get Capabilities Layer Name'),
        ),
        migrations.AlterField(
            model_name='wmslayer',
            name='custom_get_capabilities_url',
            field=models.URLField(blank=True, help_text="Alternative URL for the GetCapabilities request. Used when 'Request time from capabilities' option is checked. Leave blank to use base url.", null=True, verbose_name='Get Capabilities URL'),
        ),
    ]



from adminboundarymanager.models import AdminBoundarySettings
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from wagtail.admin.menu import MenuItem, Menu
from wagtail_modeladmin.menus import GroupMenuItem
from wagtail_modeladmin.options import ModelAdminGroup

from geomanager.admin.category import CategoryModelAdmin
from geomanager.admin.dataset import DatasetModelAdmin
from geomanager.admin.mbt_source import MBTSourceModelAdmin
from geomanager.admin.metadata import MetadataModelAdmin
from geomanager.admin.raster_file import RasterFileLayerModelAdmin, RasterFileModelAdmin, urls as raster_file_urls
from geomanager.admin.raster_style import RasterStyleModelAdmin
from geomanager.admin.raster_tile import RasterTileLayerModelAdmin, urls as raster_tile_urls
from geomanager.admin.vector_file import VectorFileLayerModelAdmin, VectorTableModelAdmin, urls as vector_file_urls
from geomanager.admin.vector_tile import VectorTileLayerModelAdmin, urls as vector_tile_urls
from geomanager.admin.wms import WmsLayerModelAdmin, urls as wms_urls
from geomanager.models import GeomanagerSettings

urls = raster_file_urls + raster_tile_urls + vector_file_urls + vector_tile_urls + wms_urls


class ModelAdminGroupWithHiddenItems(ModelAdminGroup):
    def get_submenu_items(self):
        menu_items = []
        item_order = 1
        for model_admin in self.modeladmin_instances:
            if not model_admin.hidden:
                menu_items.append(model_admin.get_menu_item(order=item_order))
                item_order += 1
        return menu_items


class GeoManagerAdminGroup(ModelAdminGroupWithHiddenItems):
    menu_label = _('Geo Manager')
    menu_icon = 'layer-group'
    menu_order = 700
    items = (
        CategoryModelAdmin,
        DatasetModelAdmin,
        MetadataModelAdmin,
        RasterFileLayerModelAdmin,
        RasterStyleModelAdmin,
        VectorFileLayerModelAdmin,
        WmsLayerModelAdmin,
        RasterTileLayerModelAdmin,
        VectorTileLayerModelAdmin,
        MBTSourceModelAdmin,
        RasterFileModelAdmin,
        VectorTableModelAdmin
    )

    def get_menu_item(self):
        if self.modeladmin_instances:
            submenu = Menu(items=self.get_submenu_items(), register_hook_name='register_geo_manager_menu_item')
            return GroupMenuItem(self, self.get_menu_order(), submenu)

    def get_submenu_items(self):
        menu_items = super().get_submenu_items()

        try:
            settings_url = reverse(
                "wagtailsettings:edit",
                args=[AdminBoundarySettings._meta.app_label, AdminBoundarySettings._meta.model_name, ],
            )
            abm_settings_menu = MenuItem(label=_("Boundary Settings"), url=settings_url, icon_name="cog")
            menu_items.append(abm_settings_menu)
        except Exception:
            pass

        boundary_loader = MenuItem(label=_("Boundary Data"), url=reverse("adminboundarymanager_preview_boundary"),
                                   icon_name="map")
        menu_items.append(boundary_loader)

        try:
            settings_url = reverse(
                "wagtailsettings:edit",
                args=[GeomanagerSettings._meta.app_label, GeomanagerSettings._meta.model_name, ],
            )
            gm_settings_menu = MenuItem(label=_("Settings"), url=settings_url, icon_name="cog")
            menu_items.append(gm_settings_menu)
        except Exception:
            pass

        return menu_items



from django.db import models
from django.urls import reverse
from django.utils.functional import cached_property
from django.utils.safestring import mark_safe
from django.utils.text import slugify
from django.utils.translation import gettext_lazy as _
from django_extensions.db.models import TimeStampedModel
from django_json_widget.widgets import JSONEditorWidget
from modelcluster.models import ClusterableModel
from wagtail.admin.panels import FieldPanel

from geomanager.constants import DEFAULT_OPEN_MAP_TILES_STYLE


class MBTSource(TimeStampedModel, ClusterableModel):
    name = models.CharField(max_length=255, verbose_name=_("name"), help_text=_("Style Name"))
    slug = models.CharField(max_length=255, unique=True, editable=False)
    file = models.FileField(upload_to="mbtiles", verbose_name=_("file"))
    use_default_style = models.BooleanField(default=True, verbose_name=_("Use default style"))
    open_map_style_json = models.JSONField(blank=True, null=True, verbose_name=_("OpenMapTiles style"),
                                           help_text=mark_safe(
                                               "OpenMapTiles JSON Style. See schema here: "
                                               "<a href='https://openmaptiles.org/schema' target='_blank' rel='noopener noreferrer'>"
                                               "https://openmaptiles.org/schema</a>"))

    panels = [
        FieldPanel('name'),
        FieldPanel('file'),
        FieldPanel('use_default_style'),
        FieldPanel('open_map_style_json', widget=JSONEditorWidget(width="100%")),
    ]

    class Meta:
        verbose_name = _("Basemap Source")
        verbose_name_plural = _("Basemap Sources")

    def __str__(self):
        return self.name

    @cached_property
    def json_style(self):
        if self.use_default_style:
            return DEFAULT_OPEN_MAP_TILES_STYLE
        if self.open_map_style_json:
            return self.open_map_style_json
        return {}

    def save(self, *args, **kwargs):
        self.slug = slugify(self.name)

        super(MBTSource, self).save(*args, **kwargs)

    @property
    def map_style_url(self):
        return reverse("style_json_gl", args=[self.slug])



# Generated by Django 4.1.10 on 2023-07-31 07:23

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('geomanager', '0011_vectorlayericon'),
    ]

    operations = [
        migrations.AddField(
            model_name='geomanagersettings',
            name='crop_raster_to_country',
            field=models.BooleanField(default=True, help_text='Crop the uploaded raster file to the country boundaries', verbose_name='Crop raster to country'),
        ),
    ]
